
-- globals {{{
local title = love.graphics.newImage"title.png"
local city = love.graphics.newImage"city.png"

local human, humanWPhone = love.graphics.newImage"human.png", love.graphics.newImage"humanWPhone.png"
local humans, humansGettingEaten = {}, {}
local humanWidth, humanHeight = human:getDimensions()

local tongue = love.graphics.newImage"tongue.png"
local tongueOrigin = { 390, 236 }

local monsterStates = { "idle", "eating", "chewing", "happy", "annoyed" }
local monsterSprites, monster = {}
do
	local fmt = "monster_%s.png"
	for _, state in ipairs( monsterStates ) do
		monsterSprites[state] = love.graphics.newImage( fmt:format( state ) )
	end
end


local sounds = {
	eating = love.audio.newSource"sounds/aahm.ogg",
	chewing = love.audio.newSource"sounds/crunch.ogg",
	happy = {},
	annoyed = {},
	cry = {},
	photo = love.audio.newSource"sounds/photo.ogg",
	photo2 = love.audio.newSource"sounds/photo2.ogg",
	twitter = love.audio.newSource"sounds/twitter.ogg",
}
for i=1, 5 do
	sounds.happy[#sounds.happy+1] = love.audio.newSource( string.format( "sounds/mmh/mmh%d.ogg", i ) )
end
for i=1, 4 do
	sounds.annoyed[#sounds.annoyed+1] = love.audio.newSource( string.format( "sounds/angry/angry%d.ogg", i ) )
end
for i=1, 5 do
	sounds.cry[#sounds.cry+1] = love.audio.newSource( string.format( "sounds/cry/cry%d.ogg", i ) )
end

local frameTimes = { 1, 1.1, 1.2, 1.3, 1.4, 1.95, }
local frameStates= { 2, 3,   1,   3,   1,   4, }
local function lerp( a, b, t )
	return a + (b - a)*t
end
local function tsize( t )
	local ctr = 0
	for _, _ in pairs( t ) do  ctr = ctr + 1  end
	return ctr
end
local function animate( t )
	if t>=2 then  return  end
	local state
	for i, time in ipairs( frameTimes ) do
		if t<time then
			state = monsterStates[frameStates[i]]
			state = (state=="happy" and not hasEaten) and "annoyed" or state
			if state=="chewing" and hasEaten then  sounds.chewing:play()  end
			if state=="eating" and hasEaten and tsize( humans )>0 then
				sounds.photo:play()
				if tsize( humans )>1 then  sounds.photo2:play()  end
			end
			monster = monsterSprites[state]
			break
		elseif i==6 then -- and t>=time
			monster, hasEaten = monsterSprites.idle
		end
	end
	if not played and t>=1.4 then
		sounds[state][soundIndex[state]]:play()
		played = true
	end
end
-- }}}

-- love callbacks {{{
function love.load()
	w, h = love.graphics.getWidth(), love.graphics.getHeight()
	winmindim = math.min( w, h )
	t, mx, my = 2, 0, 0
	love.graphics.setBackgroundColor( 255, 255, 255 )
	for i=1, 8 do
		humans[i] = math.random( w - humanWidth )
	end
	monster = monsterSprites.idle
	love.draw, love.mousepressed, hasEaten, tweeted = drawTitleScreen, titleMousepressed
end

function updateGame( dt )
	t = t + dt
	animate( t )
	if not hasEaten then
		for i, hmn in pairs( humans ) do
			humans[i] = math.min( w - humanWidth, math.max( 0, hmn + (math.random( 2 )==1 and 1 or -1)*math.random( 10 )) )
		end
	end
	if tsize( humans )==0 and monster==monsterSprites.happy then
		love.load()
	end
end

function drawTitleScreen()
	love.graphics.draw( title )
end
function drawGame()
	updateGame( love.timer.getDelta() )
	love.graphics.draw( monster )
	love.graphics.draw( city )
	if t<1 then
		local dx, dy = mx - tongueOrigin[1], my - tongueOrigin[2]
		love.graphics.setScissor( 0, tongueOrigin[2], w, h - tongueOrigin[2] )
		local distance, tongueLength = (dx^2 + dy^2)^(1/2), tongue:getWidth() -- because sprite is not vertical
		local d = math.max( 0, tongueLength - distance ) - tongue:getHeight()/4 -- empiric padding
		local t = math.cos( t*math.pi )^2
		love.graphics.draw( tongue, tongueOrigin[1], tongueOrigin[2], math.atan2( dy, dx ), 1, 1, t*tongueLength + d, tongue:getHeight()/2 )
		love.graphics.setScissor()
		for i, x in pairs( humansGettingEaten ) do
			if t>=0.99 then  humansGettingEaten[i] = nil  end
			love.graphics.draw( human, lerp( x, tongueOrigin[1] - humanWidth/2, t ), lerp( h - humanHeight, tongueOrigin[2], t ) )
		end
	end
	for i, x in pairs( humans ) do
		if mx>=x and mx<=x+humanWidth and 0.5<t and t<0.6  and monster~=monsterSprites.idle and not hasEaten then
			humansGettingEaten[i], humans[i] = x, nil
			hasEaten = true
			sounds.cry[soundIndex.cry]:play()
		else
			if hasEaten or tsize( humans )==1 then
				love.graphics.draw( humanWPhone, x, h - humanHeight, 0, x>tongueOrigin[1] and -1 or 1, 1 )
			else
				love.graphics.draw( human, x, h - humanHeight )
			end
			if tsize( humans )==1 and next( humansGettingEaten )==nil and not tweeted and t>2 then
				sounds.twitter:play()
				tweeted = true
			end
		end
	end
	if hasEaten and monster==monsterSprites.eating and tsize( humans )>0 then
		love.graphics.setColor( 255, 255, 255, math.sin(t*10)^2*80 )
		love.graphics.rectangle( "fill", 0, 0, w, h )
		love.graphics.setColor( 255, 255, 255, 255 )
	end
end
-- }}}

-- keyboard/mouse handling {{{
keys = {
	escape = love.event.quit,
}

function love.keypressed( k )
	local action = keys[k]
	if action then  return action()  end
end
function titleMousepressed()
	love.update = updateGame
	love.draw = drawGame
	love.mousepressed = gameMousepressed
end
function gameMousepressed( x, y )
	if y>2/3*h and monster==monsterSprites.idle then
		sounds.eating:play()
		soundIndex, played = { happy = math.random( #sounds.happy ), annoyed = math.random( #sounds.annoyed ), cry = math.random( #sounds.cry ) }
		mx, my = x, y
		t = 0
	end
end
-- }}}

-- vim: set fdm=marker :
